@@ -1,30 +1,42 @@
-# Copyright (C) 2022 by Jaedson Silva
-# This code is licensed under MIT license (see LICENSE.txt for details)
+# CookieDB
+# Copyright (C) 2022  Jaedson Silva
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-from .json_handler import JSONHandler
+from ._document import JSONHandler
 from .exceptions import DatabaseNotFoundError, DatabaseExistsError, NoOpenDatabaseError
 
-from typing import Union, Any
+from typing import Any
 from functools import wraps
 
 
 def required_database(method):
     @wraps(method)
-    def wrapper(ref, *args, **kwargs):
-        if ref.get_opened_database() is None:
+    def decorator(ref, *args, **kwargs):
+        if ref.checkout() is None:
             raise NoOpenDatabaseError('No open database.')
         else:
             return method(ref, *args, **kwargs)
 
-    return wrapper
+    return decorator
 
 
 class CookieDB:
     def __init__(
         self,
         key: str = None,
-        database_local: str = None,
-        autocommit: bool = False
+        database_local: str = None
     ):
         """
         Initializes the **JSONHandler** class and prepares the
@@ -40,9 +52,8 @@ class CookieDB:
         time a database method is called.
         """
 
-        self._json_handler = None
+        self._document = None
         self._open_database = None
-        self._temp_items = {}
 
         if not database_local:
             database_local = './'
@@ -52,15 +63,10 @@ class CookieDB:
 
         self._key = key
         self._database_local = database_local
-        self._autocommit = autocommit
 
-        self._json_handler = JSONHandler(self._key, self._database_local)
+        self._document = JSONHandler(self._key, self._database_local)
 
-    def _auto_commit(self):
-        if self._autocommit:
-            self.commit()
-
-    def get_opened_database(self):
+    def checkout(self):
         return self._open_database
 
     def open(self, database_name: str) -> None:
@@ -73,7 +79,7 @@ class CookieDB:
         :return: None.
         """
 
-        database_exists = self._json_handler.exists_database(database_name)
+        database_exists = self._document.exists_document(database_name)
 
         if not database_exists:
             raise DatabaseNotFoundError(f'Database {database_name} not found.')
@@ -93,58 +99,46 @@ class CookieDB:
         :return: None.
         """
 
-        if not self._json_handler.exists_database(database_name):
-            self._json_handler.create_json_database(database_name)
+        if not self._document.exists_document(database_name):
+            self._document.create_document(database_name)
         else:
             if not if_not_exists:
                 raise DatabaseExistsError(f'Database {database_name} already exists.')
 
-    @required_database
-    def commit(self) -> bool:
-        """
-        Save changes made to the database.
+    def _get_database_items(self):
+        database = self._document.get_document(self._open_database)
+        return database.get('items')
 
-        :return: Returns "True" if there were changes to commit.
-        """
-
-        if self._temp_items is None:
-            return False
-
-        self._json_handler.update_database(self._open_database, self._temp_items)
-        return False
+    def _filter_path_list(self, path_list: list) -> str:
+        path_list_filtered = [i for i in path_list if i != '']
+        return path_list_filtered
 
     @required_database
-    def create_item(self, path: Union[str, int], value: Any) -> None:
+    def add(self, path: str, value: Any) -> None:
         """
         Creates an item in the database.
         Each path separated by "/" is a key in the JSON file.
 
-        Values can be of type `str`, `int`,
-        `float`, `dict`, `list`, or `tuple`.
-
         :param path: Item path;
         :param value: Item value;
         :return: None.
         """
 
-        items = self._temp_items
-        path_list = path.split('/')
-        path_list_filtered = []
+        database_items = self._get_database_items()
+        items = database_items
 
-        for i in path_list:
-            if i != '':
-                path_list_filtered.append(i)
+        path_list = self._filter_path_list(path.split('/'))
 
-        for c, i in enumerate(path_list_filtered):
-            if c == (len(path_list_filtered) - 1):
-                items = items.setdefault(i, value)
+        for c, i in enumerate(path_list):
+            if c == (len(path_list) - 1):
+                items[i] = value
             else:
                 items = items.setdefault(i, {})
 
-        self._auto_commit()
+        self._document.update_document(self._open_database, database_items)
 
     @required_database
-    def get_item(self, path: Union[str, int]) -> Any:
+    def get(self, path: str) -> Any:
         """
         Get a database item from the path.
 
@@ -153,19 +147,31 @@ class CookieDB:
         None if nothing is found.
         """
 
-        path_list = path.split('/')
+        path_list = self._filter_path_list(path.split('/'))
         last_items = {}
 
-        database = self._json_handler.get_database(self._open_database)
-        database_items = database.get('items')
+        database_items = self._get_database_items()
 
         for i in path_list:
-            if i != '':
-                if not last_items:
-                    db_item = database_items.get(i)
-                    last_items = db_item
-                else:
-                    sub_item = last_items.get(i)
-                    last_items = sub_item
+            if not last_items:
+                db_item = database_items.get(i)
+                last_items = db_item
+            else:
+                sub_item = last_items.get(i)
+                last_items = sub_item
 
         return last_items
+
+    def delete(self, path: str) -> bool:
+        database_items = self._get_database_items()
+        df = database_items
+
+        path_list = self._filter_path_list(path.split('/'))
+
+        for c, i in enumerate(path_list):
+            if c == (len(path_list) - 1):
+                df = df.pop(i)
+            else:
+                df = df.setdefault(i, {})
+
+        self._document.update_document(self._open_database, database_items)
